@@ -23,10 +23,12 @@
 from typing import Dict
 from typing import List
 from typing import Tuple
+from typing import NamedTuple
 from typing import Iterable
 from typing import Any
 import sys
 import ctypes
+import logging
 from gi import require_version # type: ignore
 # pylint: disable=wrong-import-position
 require_version('IBus', '1.0')
@@ -34,6 +36,8 @@ from gi.repository import IBus # type: ignore
 # pylint: enable=wrong-import-position
 import itb_util
 
+LOGGER = logging.getLogger('ibus-typing-booster')
+
 # pylint: disable=invalid-name
 # pylint: disable=too-few-public-methods
 # pylint: disable=missing-class-docstring
@@ -913,112 +917,50 @@ def convert_digits_to_ascii(text: str) -> str:
     '''
     return text.translate(DIGIT_TRANS_TABLE)
 
+class TransliterationParts(NamedTuple):
+    '''
+    A named tuple containing the parts of a transliteration
+
+    committed: str        The part of the transliteration which cannot be
+                          changed anymore by adding more input, could be
+                          committed already if desired.
+    committed_index: int  The index up to which the msymbol_list input
+                          was “used up” to create the “committed” text.
+    preedit:              The transliteration of the remaining input,
+                          may still change by adding more input.
+    cursor_pos: int       The cursor position in the preedit.
+                          Counted in codepoints, not glyphs.
+                          Usually this is at the end of the preedit
+                          but an input method may move the cursor
+                          within the preedit!
+                          (I think only ja-anthy.mim actually uses this)
+    status: str           May change for some input methods to
+                          indicate a state.
+                          For example in case of ja-anthy.mim,
+                          this is 'aあ' before Henkan and changes
+                          to '漢' in Henkan mode.
+    candidates: List[str] May contain a list of candidates if the
+                          input method can produce multiple candidates.
+    candidate_show: int   0: candidates should be hidden
+                          1: candidates should be shown
+    '''
+    committed: str = ''
+    committed_index: int = 0
+    preedit: str = ''
+    cursor_pos: int = 0
+    status: str = ''
+    candidates: List[str] = []
+    candidate_show: int = 0
+
 class Transliterator:
+    # pylint: disable=line-too-long
     '''A class for transliterators using libm17n
 
     If initializing the transliterator fails, for example because a
     non-existing input method was given as the argument, a ValueError
-    is raised:
-
-    Examples:
-
-    Russian transliteration:
-
-    >>> trans = Transliterator('ru-translit')
-    >>> trans.transliterate(list('y'))
-    'ы'
-    >>> trans.transliterate(list('yo'))
-    'ё'
-    >>> trans.transliterate(list('yo y'))
-    'ё ы'
-
-    Marathi transliteration:
-
-    >>> trans = Transliterator('mr-itrans')
-    >>> trans.transliterate(list('praviN'))
-    'प्रविण्'
-    >>> trans.transliterate(list('namaste'))
-    'नमस्ते'
-
-    Hindi transliteration:
-
-    >>> trans = Transliterator('hi-itrans')
-    >>> trans.transliterate(list('namaste'))
-    'नमस्ते'
-
-    >>> trans.transliterate_parts(list('n'))
-    ('', 'न्')
-
-    >>> trans.transliterate_parts(list('n '))
-    ('न ', '')
-
-    >>> trans.transliterate_parts(list('na'))
-    ('', 'न')
-
-    >>> trans.transliterate_parts(list('nam'))
-    ('न', 'म्')
-
-    >>> trans.transliterate_parts(list('nama'))
-    ('न', 'म')
-
-    >>> trans.transliterate_parts(list('namas'))
-    ('नम', 'स्')
-
-    >>> trans.transliterate_parts(list('namast'))
-    ('नम', 'स्त्')
-
-    >>> trans.transliterate_parts(list('namaste'))
-    ('नम', 'स्ते')
-
-    >>> trans.transliterate_parts(list('namaste '))
-    ('नमस्ते ', '')
-
-    >>> trans.transliterate(list('. '))
-    '। '
-
-    Hindi-Inscript2 uses the AltGr key a lot, 'G-4' is
-    the MSymbol name for AltGr-4 and it transliterates
-    to something different than just '4':
-
-    >>> trans = Transliterator('hi-inscript2')
-    >>> trans.transliterate(['4', 'G-4'])
-    '४₹'
-
-    >>> trans = Transliterator('hi-inscript2')
-    >>> trans.transliterate(['G-p'])
-    'ज़'
-
-    AltGr-3 ('G-3') is not used though in Hindi-Inscript2.
-    Therefore, 'G-3' transliterates just as 'G-3':
-
-    >>> trans = Transliterator('hi-inscript2')
-    >>> trans.transliterate(['3', 'G-3'])
-    '३G-3'
-
-    In mr-inscript2, 'G-1' transliterates to U+200D ZERO WIDTH JOINER
-    ('\xe2\x80\x8d' in UTF-8 encoding):
-
-    >>> trans = Transliterator('mr-inscript2')
-    >>> trans.transliterate(['j', 'd', 'G-1', '/']).encode('utf-8')
-    b'\xe0\xa4\xb0\xe0\xa5\x8d\xe2\x80\x8d\xe0\xa4\xaf'
-
-    >>> trans = Transliterator('t-latn-post')
-    >>> trans.transliterate(list('gru"n'))
-    'grün'
-
-    >>> trans = Transliterator('NoIME')
-    >>> trans.transliterate(['a', 'b', 'c', 'C-c', 'G-4', 'C-α', 'G-α'])
-    'abcC-cG-4C-αG-α'
-
-    >>> trans = Transliterator('ko-romaja')
-    >>> trans.transliterate(list('annyeonghaseyo'))
-    '안녕하세요'
-
-    >>> trans = Transliterator('si-wijesekara')
-    >>> trans.transliterate(list('vksIal kjSka '))
-    'ඩනිෂ්ක නවීන් '
-
+    is raised.
     '''
+    # pylint: enable=line-too-long
     def __init__(self, ime: str) -> None:
         '''Initialize the input method to use for the transliteration
 
@@ -1054,7 +996,7 @@ class Transliterator:
     def transliterate_parts(
             self,
             msymbol_list: Iterable[str],
-            ascii_digits: bool = False) -> Tuple[str, str]:
+            ascii_digits: bool = False) -> TransliterationParts:
         '''Transliterate a list of Msymbol names
 
         :param msymbol_list: A list of strings which are interpreted
@@ -1064,15 +1006,420 @@ class Transliterator:
                              Msymbols is just joined to a single string.
         :param ascii_digits: If true, convert language specific digits
                              to ASCII digits
-        :return: The transliteration in two parts: (committed, preedit)
+        :return: The transliteration in several parts
+
+        Examples:
+
+        >>> trans = Transliterator('hi-itrans')
+        >>> parts = trans.transliterate_parts(list('n'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        'न्'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('n '))
+        >>> parts.committed
+        'न '
+        >>> parts.committed_index
+        2
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts =trans.transliterate_parts(list('na'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        'न'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('nam'))
+        >>> parts.committed
+        'न'
+        >>> parts.committed_index
+        2
+        >>> parts.preedit
+        'म्'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('nama'))
+        >>> parts.committed
+        'न'
+        >>> parts.committed_index
+        2
+        >>> parts.preedit
+        'म'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('namas'))
+        >>> parts.committed
+        'नम'
+        >>> parts.committed_index
+        4
+        >>> parts.preedit
+        'स्'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('namast'))
+        >>> parts.committed
+        'नम'
+        >>> parts.committed_index
+        4
+        >>> parts.preedit
+        'स्त्'
+        >>> parts.cursor_pos
+        4
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('namaste'))
+        >>> parts.committed
+        'नम'
+        >>> parts.committed_index
+        4
+        >>> parts.preedit
+        'स्ते'
+        >>> parts.cursor_pos
+        4
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('namaste '))
+        >>> parts.committed
+        'नमस्ते '
+        >>> parts.committed_index
+        8
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'क'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+
+        >>> trans = Transliterator('t-latn-post')
+        >>> parts = trans.transliterate_parts(list('u'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        'u'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'Latin-post'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('u"'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        'ü'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'Latin-post'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('u""'))
+        >>> parts.committed
+        'u"'
+        >>> parts.committed_index
+        3
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'Latin-post'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('u"u'))
+        >>> parts.committed
+        'ü'
+        >>> parts.committed_index
+        2
+        >>> parts.preedit
+        'u'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'Latin-post'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('üu"u'))
+        >>> parts.committed
+        'üü'
+        >>> parts.committed_index
+        3
+        >>> parts.preedit
+        'u'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'Latin-post'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+
+        >>> trans = Transliterator('t-rfc1345')
+        >>> parts = trans.transliterate_parts(list('&'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        '&'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('&C'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        '&C'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('&Co'))
+        >>> parts.committed
+        '©'
+        >>> parts.committed_index
+        3
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('&f'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        '&f'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('&ff'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        'ﬀ'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('&ffi'))
+        >>> parts.committed
+        'ﬃ'
+        >>> parts.committed_index
+        4
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'RFC1345'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+
+        >>> trans = Transliterator('t-lsymbol')
+        >>> parts = trans.transliterate_parts(list('/:)'))
+        >>> parts.committed
+        ''
+        >>> parts.committed_index
+        0
+        >>> parts.preedit
+        '☺️'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        ['☺️', '😃', '😅', '😆', '😉', '😇', '😂', '😏', '😛', '😜', '😝', '😋', '😉', '💏', '💋', '😍', '😘', '😚', '😽', '😻']
+        >>> parts.candidate_show
+        1
+        >>> parts = trans.transliterate_parts(list('a'))
+        >>> parts.committed
+        'a'
+        >>> parts.committed_index
+        1
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+        >>> parts = trans.transliterate_parts(list('a/'))
+        >>> parts.committed
+        'a'
+        >>> parts.committed_index
+        1
+        >>> parts.preedit
+        '/'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        ['/']
+        >>> parts.candidate_show
+        1
+        >>> parts = trans.transliterate_parts(list('a/:'))
+        >>> parts.committed
+        'a'
+        >>> parts.committed_index
+        1
+        >>> parts.preedit
+        '/:'
+        >>> parts.cursor_pos
+        2
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        ['/:']
+        >>> parts.candidate_show
+        1
+        >>> parts = trans.transliterate_parts(list('a/:('))
+        >>> parts.committed
+        'a'
+        >>> parts.committed_index
+        1
+        >>> parts.preedit
+        '😢'
+        >>> parts.cursor_pos
+        1
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        ['😢', '😩', '😡', '😭', '😪', '🙈', '🙊', '🙉']
+        >>> parts.candidate_show
+        1
+        >>> parts = trans.transliterate_parts(list('a/:(b'))
+        >>> parts.committed
+        'a😢b'
+        >>> parts.committed_index
+        5
+        >>> parts.preedit
+        ''
+        >>> parts.cursor_pos
+        0
+        >>> parts.status
+        'lsymbol'
+        >>> parts.candidates
+        []
+        >>> parts.candidate_show
+        0
+
+        For a test transliterating parts using 'ja-anthy' see 'tests/test_m17n_translit.py'.
         '''
         if not isinstance(msymbol_list, list):
             raise ValueError('Argument of transliterate() must be a list.')
         if self._dummy:
-            return (''.join(msymbol_list), '')
+            return TransliterationParts(committed=''.join(msymbol_list),
+                                       committed_index=len(msymbol_list))
         libm17n__minput_reset_ic(self._ic) # type: ignore
         committed = ''
+        committed_index = 0
         preedit = ''
+        candidates: List[str] = []
         for index, symbol in enumerate(msymbol_list):
             if len(symbol) == 1 and not itb_util.is_ascii(symbol):
                 symbol = IBus.keyval_name(IBus.unicode_to_keyval(symbol))
@@ -1090,8 +1437,10 @@ class Transliterator:
                     self._ic, _symbol, ctypes.c_void_p(None), _mt)
                 if libm17n__mtext_len(_mt) > 0: # type: ignore
                     committed += mtext_to_string(_mt)
+                    committed_index = index
                 if retval:
                     committed += msymbol_list[index]
+                    committed_index = index + 1
         try:
             if (self._ic.contents.preedit_changed
                 and
@@ -1101,6 +1450,43 @@ class Transliterator:
         except Exception as error: # pylint: disable=broad-except
             # This should never happen:
             raise ValueError('Problem accessing preedit') from error
+        plist = self._ic.contents.candidate_list
+        while bool(plist):  # NULL pointers have a False boolean value
+            key = libm17n__mplist_key(plist) # type: ignore
+            if not bool(key):
+                break
+            key_name = libm17n__msymbol_name(key.contents) # type: ignore
+            if key_name == b'mtext':
+                characters = mtext_to_string(
+                    ctypes.cast(libm17n__mplist_value(plist), # type: ignore
+                        ctypes.POINTER(libm17n__MText)))
+                candidates += list(characters)
+            elif key_name == b'plist':
+                candidate_plist = ctypes.cast(
+                    libm17n__mplist_value(plist), # type: ignore
+                    ctypes.POINTER(libm17n__MPlist))
+                while True:
+                    candidate_plist_key = libm17n__mplist_key( # type: ignore
+                        candidate_plist)
+                    if not bool(candidate_plist_key):
+                        break
+                    candidate_plist_key_name = libm17n__msymbol_name( # type: ignore
+                        candidate_plist_key.contents)
+                    if candidate_plist_key_name != b'mtext':
+                        break
+                    candidate = mtext_to_string(
+                        ctypes.cast(
+                            libm17n__mplist_value(candidate_plist), # type: ignore
+                            ctypes.POINTER(libm17n__MText)))
+                    candidates.append(candidate)
+                    candidate_plist = libm17n__mplist_next( # type: ignore
+                        candidate_plist)
+            else:
+                break
+            plist = libm17n__mplist_next(plist) # type: ignore
+        cursor_pos = self._ic.contents.cursor_pos
+        status = mtext_to_string(self._ic.contents.status)
+        candidate_show = self._ic.contents.candidate_show
         # From the m17n-lib documentation:
         #
         # The minput_reset_ic () function resets input context $IC by
@@ -1125,10 +1511,69 @@ class Transliterator:
         _symbol = libm17n__msymbol(b'nil') # type: ignore
         _retval = libm17n__minput_filter( # type: ignore
             self._ic, _symbol, ctypes.c_void_p(None))
+        if committed and not preedit:
+            committed_index = len(msymbol_list)
+        # Some Chinese input methods and some Vietnamese input methods
+        # for Chinese characters sometimes have “candidates == []” but
+        # at the same time “candidate_show == 1”:
+        #
+        # For example zh-cangjie.mim contains:
+        #
+        #  (map
+        #   ("a" ("日曰"))
+        #   ("aa" ("昌昍"))
+        #   ("aaa" ?晶)
+        #   ("aaaa" ("𣊫𣊭"))
+        #   [...]
+        #
+        # The ?晶 produces “candidates == []”, ("晶") would
+        # produce “candidates == ['晶']”
+        #
+        # In that case typing `a` and `aa` produces candidates but
+        # `aaa` does not and `aaaa` produces candidates again.  That
+        # is bad because Typing Booster behaves differently when there
+        # are candidates and when there are not.  So it would behave
+        # inconsistently while typing `aaaa` if `aaa` suddenly has no
+        # candidates and suddenly shows a “normal” typing booster
+        # completion lookup table.
+        #
+        # Other input methods which have the same problem are
+        # zh-tonepy, vi-nomtelex, vi-nomvni, ...
+        #
+        # So copy the preedit into candidates if they are empty and
+        # candidate_show is one for more consistent behaviour (in my
+        # opinion):
+        if preedit and not candidates and candidate_show:
+            candidates = [preedit]
+        if self._language == 'ja' and self._name == 'anthy':
+            # ja-anthy seems to produce a lot of useless geta marks '
+            # 〓'.  It also produces sometimes useless empty
+            # candidates and sometimes (when transliterating `1 ` even
+            # preedit='', cursor_pos=1. Try to fix this here:
+            if preedit != '下駄':
+                candidates = [
+                    candidate
+                    for candidate in candidates
+                    if candidate and '〓' not in candidate]
+            if not preedit and candidates:
+                preedit = candidates[0]
+                cursor_pos = len(preedit)
         if not ascii_digits:
-            return (committed, preedit)
-        return (convert_digits_to_ascii(committed),
-                convert_digits_to_ascii(preedit))
+            return TransliterationParts(committed=committed,
+                                       committed_index=committed_index,
+                                       preedit=preedit,
+                                       cursor_pos=cursor_pos,
+                                       status=status,
+                                       candidates=candidates,
+                                       candidate_show=candidate_show)
+        return TransliterationParts(
+            committed=convert_digits_to_ascii(committed),
+            committed_index=committed_index,
+            preedit=convert_digits_to_ascii(preedit),
+            cursor_pos=cursor_pos,
+            status=status,
+            candidates=candidates,
+            candidate_show=candidate_show)
 
     def transliterate(self, msymbol_list: Iterable[str], ascii_digits: bool = False) -> str:
         '''Transliterate a list of Msymbol names
@@ -1141,9 +1586,80 @@ class Transliterator:
         :param ascii_digits: If true, convert language specific digits
                              to ASCII digits
         :return: The transliteration in one string
+
+        Examples:
+
+        Russian transliteration:
+
+        >>> trans = Transliterator('ru-translit')
+        >>> trans.transliterate(list('y'))
+        'ы'
+        >>> trans.transliterate(list('yo'))
+        'ё'
+        >>> trans.transliterate(list('yo y'))
+        'ё ы'
+
+        Marathi transliteration:
+
+        >>> trans = Transliterator('mr-itrans')
+        >>> trans.transliterate(list('praviN'))
+        'प्रविण्'
+        >>> trans.transliterate(list('namaste'))
+        'नमस्ते'
+
+        Hindi transliteration:
+
+        >>> trans = Transliterator('hi-itrans')
+        >>> trans.transliterate(list('namaste'))
+        'नमस्ते'
+        >>> trans.transliterate(list('. '))
+        '। '
+
+        Hindi-Inscript2 uses the AltGr key a lot, 'G-4' is
+        the MSymbol name for AltGr-4 and it transliterates
+        to something different than just '4':
+
+        >>> trans = Transliterator('hi-inscript2')
+        >>> trans.transliterate(['4', 'G-4'])
+        '४₹'
+
+        >>> trans = Transliterator('hi-inscript2')
+        >>> trans.transliterate(['G-p'])
+        'ज़'
+
+        AltGr-3 ('G-3') is not used though in Hindi-Inscript2.
+        Therefore, 'G-3' transliterates just as 'G-3':
+
+        >>> trans = Transliterator('hi-inscript2')
+        >>> trans.transliterate(['3', 'G-3'])
+        '३G-3'
+
+        In mr-inscript2, 'G-1' transliterates to U+200D ZERO WIDTH JOINER
+        ('\xe2\x80\x8d' in UTF-8 encoding):
+
+        >>> trans = Transliterator('mr-inscript2')
+        >>> trans.transliterate(['j', 'd', 'G-1', '/']).encode('utf-8')
+        b'\xe0\xa4\xb0\xe0\xa5\x8d\xe2\x80\x8d\xe0\xa4\xaf'
+
+        >>> trans = Transliterator('t-latn-post')
+        >>> trans.transliterate(list('gru"n'))
+        'grün'
+
+        >>> trans = Transliterator('NoIME')
+        >>> trans.transliterate(['a', 'b', 'c', 'C-c', 'G-4', 'C-α', 'G-α'])
+        'abcC-cG-4C-αG-α'
+
+        >>> trans = Transliterator('ko-romaja')
+        >>> trans.transliterate(list('annyeonghaseyo'))
+        '안녕하세요'
+
+        >>> trans = Transliterator('si-wijesekara')
+        >>> trans.transliterate(list('vksIal kjSka '))
+        'ඩනිෂ්ක නවීන් '
         '''
-        (committed, preedit) = self.transliterate_parts(msymbol_list, ascii_digits)
-        return committed + preedit
+        transliteration_parts = self.transliterate_parts(
+            msymbol_list, ascii_digits)
+        return transliteration_parts.committed + transliteration_parts.preedit
 
     def get_variables(self) -> List[Tuple[str, str, str]]:
         # pylint: disable=line-too-long
@@ -1345,6 +1861,9 @@ class Transliterator:
         return
 
 if __name__ == "__main__":
+    LOG_HANDLER = logging.StreamHandler(stream=sys.stderr)
+    LOGGER.setLevel(logging.DEBUG)
+    LOGGER.addHandler(LOG_HANDLER)
     import doctest
-    (FAILED, _ATTEMPTED, *_EXTRAS) = doctest.testmod()
+    (FAILED, _ATTEMPTED) = doctest.testmod()
     sys.exit(FAILED)
